/*

Ultra Dune II,

Idea from "Super Dune II", which let you play the other 3 houses, this SUPER dune 2, will
set dune 2 in 640X480 mode and better AI,  but same graphics and such! 

Programmer : Stefan Hendriks
Version    : 0.00

*/


#include <time.h>
#include <stdlib.h>
#include <iostream.h>		// Input Output routines
#include <allegro.h>		// Include Allegro Lib

#include "graph01.h"		// Terrain data file		
#include "graphcm.h"		// Common graphics data file
#include "structs.h"            // Structure Graphics
                                // Format of pictures is:
                                // Atreides / Harkonnen / Ordos / Sardaukar
                                // Fremen / Mercenary
                                // This is also for units

#include "defints.h"            // Definitions of Structs and other stuff

void clear_rest_u(int start);				// Clears list from start (UNITS)
void draw_icon_uni(int icn, int ex);                    // Draws an icon of a unit
char *name_unit(int id);
void handle_walls(int cll);                             // Handle walls
void handle_lists();					// Handle lists
int convert_icon2str(int icon);                         // Returns structure ID nr
int place_structure(int xsize, int ysize);              // Size
void add_blooms(int amount);                            // Add amount of spice n blooms
void draw_icon_str(int icn, int ex);                    // Draws an icon of a structure
void do_building();                                     // Do all the build stuff
int is_building(int id);                                // Returns 0 for NO, != 0 means is building
void add_builder(int id, int typeU, int typeB, int pl); // Add a builder
char *name_structure(int id);                           // Gives NAME of structure
void unselect_all();                                    // Unselects all Units and Structures
void clear_occupieds();                                 // Clears occupied cells
void die_all_structures();				// All structures are dead
void set_houses();					// Reset values of houses
int x_cell(int cell);					// Give X of a cell
int y_cell(int cell);					// Give Y of a cell
int tile_nr(int cell);					// Give Tile NR of a cell
int make_cell(int x, int y);				// Make a cell out of X and Y
void do_sidebar();					// Do sidebar stuff
int install_game();					// Install game
void do_mouse();						// Do mouse stuff
void draw_terrain();					// Draw terrain on screen
void make_random_map();					// Creates a random map
int rnd(int range);					// Returns a random figure
void make_spot(int cell, int tpe);			// Creates a spot of TPE 
int fix_cell(int cell);					// Returns cell fixed, if nescesary
void thick_spots(int tpe);				// Thick spots of TPE
int side4(int cell, int tile);                          // Check at 4 sides for specific tile
void clear_terrain(int tile);				// Clear terrain with TILE
void smooth_terrain();					// Smooths terrain
int is_tile(int cl1, int cl2);                          // Compares 2 cells and returns TRUE when
                                                        // both cells have the same tile
int mouse_cell();                                       // Returns the cell where the mouse is
                                                        // pointing at.
int on_left(int cell, int tile);			// At left of CELL is TILE?
int on_right(int cell, int tile);			// At right of CELL is TILE?
int on_up(int cell, int tile);                          // At up of CELL is TILE?
int on_down(int cell, int tile);			// At down of CELL is TILE?
void draw_minimap();					// Draws minimap, and handles the actions
                                                        // of it, such as clicking on it etc

void clear_rest_s(int start);				// Clears list from start (STRUCTURES)
int sort_id(int id);					// Returns sort of building
void set_maxes();                                       // Will set max scroll variables

/*

                        VARIABLES

*/



mp map[4096];						// Amount of cells, store in data
pl players[7];						// 6 Houses
Build builders[250];                                    // 250 Builds at a time
// Builds are chained at structure ID!, so build[1] means that structure[1]
// is building!
Structr structure[250];					// 250 Structures

int x1 = -1, y1 = -1, x2, y2;				// For BOX
int mouse_draw_box = 0;					// Draws box?

int icons_struct[30];                                   // Structures list
int icons_units[20];                                    // Units list (vehicles)
int icons_air[5];                                       // Air units list
int icons_inf[5];							  // Infantry
int icons_star[20];						  // Starport

int building_w = 0;                                     // the Width for placing
int building_h = 0;                                     // Height

int techlevel = 3;					// Tech level in game
int mouse_status;                                       // 0 = Normal
                                                        // 1 = Move
                                                        // 2 = Attack
                                                        // 3 = Boxdrawing
int smooth_fase = -1;                                   // Smooth fase
int ingame_info;                                        // 0 = Normal
                                                        // 1 = Placing structure
int placing_good;                                       // 0 = NO, 1 = YES                                                        
int sidebar_info;                                       // Info that
                                                        // must be displayed.
                                                        // E.G = CONST / NONE / UNIT
int sidebar_info_id;                                    // ID of building we selected and is in the sidebar
int max_scroll_building;                                // Max for buildings list
int max_scroll_unit;                                    // Max for units
int max_scroll_air;                                     // Max for air units
int sidebar_scroll;					// Sidebar scroll
int MX, MY;                                             // Map X and Y
bool game_running = true;				// Playing game
DATAFILE *str;                                          // Structures Data file
DATAFILE *graph01;					// Terrain data file
DATAFILE *graphcm;					// Common graphics
BITMAP *page;						// Drawing page
PALETTE pal;                                            // Palette for screenshot

/*
Color definitions
*/
int color_slab;                 // Color of a slab on the minimap
int color_rocky;		// Minimap ROCKY
int color_sandy;		// Minimap SANDY
int color_white;		// White
int color_black;		// Black
int color_greenish;             // Yellow and green mixed
int color_yellow;               // Yellow (credits out)
int color_green;                // Green (Credits IN and Ordos)
int color_red;                  // Red (Harkonnen)
int color_blue;                 // Blue (Atreides)
int color_purple;               // Purple (Sardaukar)
int color_brown;                // Brown (Mercenary)
int color_spicy;                // Minimap SPICY



void clear_rest_u(int start)
{
int i;
icons_units[start] = -1;
for (i = start; i <= 20; i++)
	{
        icons_units[i] = -1;
	}

}




void draw_icon_uni(int icn, int ex)           // Draws an icon of a Unit
{
int icnx = 0;
int icny = 0;

 if (icn >= 0)
  { 
      if (icn > 9 )
          {
          icny += 48;
          icn -= 10;
          }        
      if (icn > 9 )
          {
          icny += 48;
          icn -= 10;
          }        

         icnx = icn * 64;
                  
         blit((BITMAP *)graphcm[IconsU].dat, page, icnx, icny, 560, 60 + (63 * ex), 64, 48);
  }
}





char *name_unit(int id)
{
 char *nam = "Unknown";
 switch (id)
 {
 case ICON_Upgrade   : nam =  "Upgrade        $300"; break;
 case ICON_Trike	   : nam =  "Trike	    $200"; break;
 case ICON_Quad	   : nam =  "Quad		    $300"; break;
 case ICON_LTank     : nam =  "Light Tank	    $400"; break;
 case ICON_HTank     : nam =  "Siege Tank	    $600"; break;
 case ICON_Launcher  : nam =  "Rocket Launcher$500"; break;
 case ICON_Harvester : nam =  "Harvester      $300"; break;
 case ICON_Devastator: nam =  "Devastator     $800"; break;
 case ICON_Mcv	   : nam =  "MCV           $1000"; break;
 case ICON_Troopers  : nam =  "Troopers       $200"; break;
 case ICON_Trooper   : nam =  "Trooper        $100"; break;
 case ICON_STank     : nam =  "Sonic Tank     $700"; break;
 }
 return nam;
}


void set_maxes()
{
int i;
int max_b = 0;
int max_u = 0;
int max_a = 0;
int max_i = 0;
 for (i = 0; i <= 30; i++)
  {
	if (icons_struct[i] >= 0)
	   max_b++;				//Max amount of BUILDINGS

        
        if (icons_units[i] >= 0 && i< 20)
	   max_u++;				//Max amount of UNITS

        if (icons_air[i] >=  0 && 1 < 5)
	   max_a++;				//Max amount of AIRBORNS

  }

max_scroll_building = max_b - 4;	//Scrolling is 4 icons so MAX-4
max_scroll_unit = max_u - 4;            //Same here
max_scroll_air = max_a - 4;             //and here...
}



void handle_walls(int cll)
{
        int cell = cll;
        int l = cell - 1, r = cell + 1, b = cell - 64, o = cell + 64;
        int boven = 0, onder = 0, links = 0, rechts = 0;
        l = fix_cell(l);
        o = fix_cell(o);
        r = fix_cell(r);
        b = fix_cell(b);

        if (map[l].tile >= 35 && map[l].tile <= 47)
         links = 1;

        if (map[r].tile >= 35 && map[r].tile <= 47)
         rechts = 1;

        if (map[o].tile >= 35 && map[o].tile <= 47)
         onder = 1;

        if (map[b].tile >= 35 && map[b].tile <= 47)
         boven = 1;

        int cur_wall = 36;


        // Splits

        if (links == 1 && rechts == 1 && boven == 1 && onder == 1)
                     cur_wall = 47;

        if (links == 1 && rechts == 1 && boven == 0 && onder == 1)
                     cur_wall = 43;

        if (links == 1 && rechts == 1 && boven == 1 && onder == 0)
                     cur_wall = 44;

        if (links == 0 && rechts == 1 && boven == 1 && onder == 1)
                     cur_wall = 41;

        if (links == 1 && rechts == 0 && boven == 1 && onder == 1)
                     cur_wall = 42;

        // Straight

        if (links == 1 && rechts == 1 && boven == 0 && onder == 0)
                     cur_wall = 36;

        if (links == 0 && rechts == 1 && boven == 0 && onder == 0)
                     cur_wall = 36;

        if (links == 1 && rechts == 0 && boven == 0 && onder == 0)
                     cur_wall = 36;

        if (links == 0 && rechts == 0 && boven == 1 && onder == 1)
                     cur_wall = 45;

        if (links == 0 && rechts == 0 && boven == 1 && onder == 0)
                     cur_wall = 45;

        if (links == 0 && rechts == 0 && boven == 0 && onder == 1)
                     cur_wall = 45;

        // Corners

        if (links == 1 && rechts == 0 && boven == 1 && onder == 0)
                     cur_wall = 40;

        if (links == 1 && rechts == 0 && boven == 0 && onder == 1)
                     cur_wall = 37;

        if (links == 0 && rechts == 1 && boven == 0 && onder == 1)
                     cur_wall = 35;

        if (links == 0 && rechts == 1 && boven == 1 && onder == 0)
                     cur_wall = 38;

        map[cell].tile = cur_wall;
}


void handle_lists()
{
// Ok check for stuff the player has
int list_index1 = 0;
int have_windtrap = players[Human].have_powers;
int have_refinery = players[Human].have_refineries;
int have_heavy_fact = players[Human].have_heavys;
int have_light_fact = players[Human].have_lights;
int have_radar = players[Human].have_radars;
int house = players[Human].house;

int upgrade1 = players[Human].upgrade_4slab;
int upgrade2 = players[Human].upgrade_4quad;

clear_rest_s(0);	// Clear all structures

/*

					STRUCTURES LISTS!

*/

        if (techlevel >= 4 && upgrade1 == 0)
	{
	icons_struct[list_index1] = ICON_Upgrade;
	list_index1++;
	}


        if (techlevel >= 6 && upgrade1 == 1 && have_radar > 0)
	{
	icons_struct[list_index1] = ICON_Upgrade;
	list_index1++;
	}

        if (techlevel >= 4)
	{
	icons_struct[list_index1] = ICON_Wall;
	list_index1++;
	}

        // Default you have:
        icons_struct[list_index1] = ICON_Slab1;        // Slab 1
	list_index1++;

        // If upgraded then we have a 4 SLAB icon here
	if (upgrade1 >= 1)
		{
		icons_struct[list_index1] = ICON_Slab4;
		list_index1++;
		}

        icons_struct[list_index1] = ICON_Windtrap;     // Windtrap
	list_index1++;

	if (have_windtrap > 0)
		{
	        icons_struct[list_index1] = ICON_Refinery;     // Refinery
        	list_index1++;

			if (techlevel >= 2)
			{
			icons_struct[list_index1] = ICON_Radar; // Radar
			list_index1++;		
			}

		}



	// Most buildings are only available when you have build once a refinery
	if (have_refinery > 0)
		{
		icons_struct[list_index1] = ICON_Silo; // Silo
		list_index1++;

		// When we have a radar and tech is higher than 5 we can build a starport
		// and a hi-tech
		if (techlevel >= 5 && have_radar > 0)
			{
			icons_struct[list_index1] = ICON_Hitech;
			list_index1++;
			icons_struct[list_index1] = ICON_Starport;
			list_index1++;
			}


		// If we are NOT harkonnen and NOT Sardaukar
		// And tech is 1 or higher than we can build barracks
		if (techlevel >= 1 && house != Harkonnen && house != Sardaukar)
			{
			icons_struct[list_index1] = ICON_Barrack; // Silo
			list_index1++;		
			}

		// We can build a wor if we ARE Harkonnen OR Sardaukar
		if (techlevel >= 1 && (house == Harkonnen || house == Sardaukar) && have_radar > 0)
			{
			icons_struct[list_index1] = ICON_Wor; // WOR!
			list_index1++;		
			}

		// All houses can build a light factory from tech level 2 or higher
		if (techlevel >= 2)
			{
			icons_struct[list_index1] = ICON_Light_Factory; // Silo
			list_index1++;		
			}

		// All houses can build a rocket turret when further upgraded, have build 1 or
		// more radars and techlevel is or higher than 6
		if (techlevel >= 6 && have_radar > 0 && upgrade1 >= 2)
			{
			icons_struct[list_index1] = ICON_Rturret;
			list_index1++;
			}

		// All houses can build a gun turret at tech 5
		if (techlevel >= 5)
			{
			icons_struct[list_index1] = ICON_Gturret;
			list_index1++;
			}

		}	// END REFINERY

	// We can build a heavy factory when we have a light factory and we are higher
	// with tech from 4. Btw we need a radar!
	if (have_light_fact > 0 && techlevel >= 4 && have_radar > 0)
		{
		icons_struct[list_index1] = ICON_Heavy_Factory;
		list_index1++;
		}

	// Once we have a heavy factory we can build... Repair and IX!
	if (have_heavy_fact > 0 && techlevel >= 7)
		{
		icons_struct[list_index1] = ICON_Repair;
		list_index1++;
		icons_struct[list_index1] = ICON_Ix;
		list_index1++;
		}

        if (have_windtrap > 0 && have_refinery > 0 && techlevel >= 8)
         {
          icons_struct[list_index1] = ICON_Palace;
          list_index1++;
         }



// The UNITS list, depends on what we have selected
// The ID will give us the needed information.

int id = sidebar_info_id;       // ID
// We don't have to worry about human or something
// since it will not be drawn anyway, so just check what kind
// of structure
int tpe_structure = structure[id].tpe;


clear_rest_u(0);	// Clear all Units


list_index1 = 0;	// Start over with index


// If we have selected a LIGHT FACTORY:
if (tpe_structure == ST_LightFact)
 {

        if (have_light_fact > 0)
        {
         if (techlevel >= 3)
                {
                icons_units[list_index1] = ICON_Upgrade;
                list_index1++;
                }

        icons_units[list_index1] = ICON_Trike;
        list_index1++;
        }


 }


/*

					UNITS LISTS!
                                    Handle Units lists...
*/


/*

					AIRBORN LISTS!
*/

set_maxes();		// Set maxes of lists...
}

int convert_icon2str(int icon)
{
 switch (icon)
 {
	// ICON		= 	STRUCTURE

        // gens
  case ICON_Slab1         : return ST_slab1;     break;
  case ICON_Slab4         : return ST_slab4;     break;
  case ICON_Wall          : return ST_Wall;      break;
        // structures
  case ICON_Windtrap      : return ST_Windtrap;  break;
  case ICON_Refinery      : return ST_Refinery;  break;
  case ICON_Silo          : return ST_Silo;      break;
  case ICON_Barrack       : return ST_Barrack;   break;
  case ICON_Radar         : return ST_Radar;     break;
  case ICON_Light_Factory : return ST_LightFact; break;
  case ICON_Heavy_Factory : return ST_HeavyFact; break;
  case ICON_Starport      : return ST_Starport;  break;
  case ICON_Ix            : return ST_IX;        break;
  case ICON_Wor           : return ST_Wor;       break;
  case ICON_Hitech        : return ST_Hitech;    break;
  case ICON_Gturret       : return ST_GunTurret; break;
  case ICON_Rturret       : return ST_RocketTurret; break;
  case ICON_Repair        : return ST_Repair;    break;

 }

return 0;
}

int place_structure(int xsize, int ysize)
{
 // Place a structure
 // Ok place a rectangle in specific color
 // Xsize = Width, Ysize = Height
 int color = color_white;       // We hope to have an succesfull placement
 int sizex = 16 * xsize;
 int sizey = 16 * ysize;
 int cx,cy;     // Counter for X and Y
 int cllx, clly;
 int mc = mouse_cell();
 if (mc < 0)
        return 5;       // Not on battle field

 cllx = x_cell(mc);
 clly = y_cell(mc);
 if (mouse_y + ysize >= 480)
        return 4;       // Out of range

 if (mouse_x + xsize >= 500)
        return 4;       // Out of range

 // Now check if we don't want to place uppon structures or units
 int flag = 1;  // Standard we DON't find anything, so we can place it
 int cll;
 int whatweplace = builders[sidebar_info_id].type_b;	// Holds ICON information

 for (cx = cllx; cx < (cllx + xsize); cx++)
  {
   for (cy = clly; cy < (clly + ysize); cy++)
    {
        cll = make_cell(cx,cy);
     if (map[cll].tile == 18)
        flag++;

     if (map[cll].occupied_s > -1 && whatweplace != ICON_Slab4)
        flag = 0;

     if ((map[cll].tile <= 0) || (map[cll].tile > 16) && map[cll].tile != 18)
        flag = 0;


    }
  }
// Draw rectangle
if (flag == 0)
        color = color_red;

if (flag == 1)
        color = color_yellow;           // YELLOW!

if (flag == 2)
        color = color_greenish;         // LITTLE MORE GREEN!

if (flag >= 3)
        color = color_green;            // YEAH! GREEN!!!!

cllx = cllx - MX;
clly = clly - MY;
rect(page, cllx * 16, clly *16 , cllx * 16 + sizex-1, clly * 16 + sizey-1, color);

return flag;    // What will it be?
}


void draw_icon_str(int icn, int ex)           // Draws an icon of a structure
{
int icnx = 0;
int icny = 0;

 if (icn >= 0)
  { 
      if (icn > 9 )
          {
          icny += 48;
          icn -= 10;
          }        
      if (icn > 9 )
          {
          icny += 48;
          icn -= 10;
          }        

         icnx = icn * 64;
                  
         blit((BITMAP *)graphcm[Icons].dat, page, icnx, icny, 560, 60 + (63 * ex ), 64, 48);
  }
}

void add_builder(int id, int typeU, int typeB, int pl)
{
int price = 100;
int slow = 0;

if (typeB == ICON_Palace)
        {
         price = 1000;
         slow = 5;
        }



if (typeB == ICON_Gturret)
        {
         price = 200;
	 slow = 1;
        }

if (typeB == ICON_Rturret)
        {
         price = 300;
	 slow = 3;
        }


if (typeB == ICON_Starport)
        {
         price = 500;
	 slow = 3;
        }

if (typeB == ICON_Upgrade)
        {
         price = 300;
	 slow = 1 + players[pl].upgrade_4slab;
        }

if (typeB == ICON_Windtrap)
        {
         price = 400;
        }

if (typeB == ICON_Refinery)
        {
         price = 600;
         slow = 1;
        }

if (typeB == ICON_Slab4)
        {
         price = 200;
        }

if (typeB == ICON_Silo)
        {
         price = 200;
        }

if (typeB == ICON_Barrack)
        {
         price = 400;
        }

if (typeB == ICON_Radar)
        {
         price = 400;
         slow = 3;
        }

if (typeB == ICON_Light_Factory)
        {
         price = 400;
         slow = 2;
        }

if (typeB == ICON_Heavy_Factory)
        {
         price = 600;
         slow = 4;
        }

if (typeB == ICON_Ix)
        {
         price = 500;
         slow = 5;
        }

if (typeB == ICON_Hitech)
        {
         price = 400;
         slow = 5;
        }

if (typeB == ICON_Repair)
        {
         price = 700;
        }

if (typeB == ICON_Wall)
        {
         price = 100;
        }


        builders[id].dead_flag = 1;                      // Lives
        builders[id].type_u = typeU;                     // Unit to build
        builders[id].type_b = typeB;                     // Str to build
        builders[id].topay = price;                        // Price to pay
        builders[id].counter = 0;                        // Counter
        builders[id].slowdowns = slow;                      // Slowdown
        builders[id].percent = 0;                        // Percent
        builders[id].player = pl;                        // Belongs to player.
}

int is_building(int id)
{
 if (builders[id].dead_flag != 0)
        return 1;       // Yes!

return 0;               // Nope 
}


char *name_structure(int id)
{
 char *nam = "Unknown";
 switch (id)
 {
 case ICON_Upgrade : nam =  "Upgrade        $300"; break;
 case ICON_Slab1 : nam =    "Small Concrete $100"; break;
 case ICON_Slab4 : nam =    "Large Concrete $200"; break;
 case ICON_Windtrap : nam = "Windtrap       $400"; break;
 case ICON_Refinery : nam = "Refinery       $600"; break;
 case ICON_Silo : nam =     "Spice Silo     $200"; break;
 case ICON_Barrack : nam =  "Barracks       $400"; break;
 case ICON_Radar : nam =    "Radar          $400"; break;
 case ICON_Light_Factory: nam = "Light Factory  $400"; break;
 case ICON_Heavy_Factory : nam = "Heavy Factory  $600"; break;
 case ICON_Starport : nam = "Starport       $500"; break;
 case ICON_Rturret : nam =  "Rocket Turret  $300"; break;
 case ICON_Gturret : nam =  "Gun Turret     $200"; break;
 case ICON_Wall : nam =     "Concrete Wall  $100"; break;
 case ICON_Hitech : nam =   "High-Tech      $500"; break;
 case ICON_Repair : nam =   "Repair Station $700"; break;
 case ICON_Ix     : nam =   "House of IX    $500"; break;
 case ICON_Wor     : nam =  "WOR            $400"; break;
 case ICON_Palace  : nam =  "Palace        $1000"; break;
 }
 return nam;

}


void clear_occupieds()
{
int i; // loop control
for (i = 0; i <= 4095; i++)
 {
 map[i].occupied_s = -1;
 map[i].occupied_u = -1;
 }

}


void die_all_structures()
{
int i; // loop control
for (i = 0; i < 250;i++)
 {
  structure[i].dead_flag = 0;	// 0 means DEAD, != 0 is ALIVE
  builders[i].dead_flag = 0;    // Same here 
 }

}


int add_structure(int player, int tpe, int cell, int flag)
{
// Returns 0, SUCCES
// -1, No more available
int i;
int res;
int hpstatus = 100;
int hpget = 100;
res = -1;
for (i = 0; i < 250; i++)
	{
        if (structure[i].dead_flag != 1)
		{
                res = i;
		break;
		}
	}

if (res == -1)
        return 1;

// If we place a slab we immidiatly
// do this and skip the coming code:
if (tpe == ST_slab1 || tpe == ST_slab4)
        {
         // 1 Slab or 4 Slabs
         map[cell].tile = 18;

         if (tpe == ST_slab4)
                {
                 map[cell + 1].tile = 18;
                 map[cell + 64].tile = 18;
                 map[cell + 65].tile = 18;
                }
        return 0;
        }

// If we place a wall, we need to check first before we put that wall
if (tpe == ST_Wall)
	{
        map[cell].tile = 36; 
	return 0;	// Succes
	}

if (tpe == ST_ConstructionYard)
  hpstatus = 300;


if (tpe == ST_GunTurret)
	hpstatus = 200;

if (tpe == ST_RocketTurret)
	hpstatus = 300;


if (tpe == ST_Refinery)
	{
	hpstatus = 200;
 	players[player].have_refineries++;
	}

if (tpe == ST_Silo)
 hpstatus = 75;

if (tpe == ST_Radar)
	{
	hpstatus = 300;
	players[player].have_radars++;
	}

if (tpe == ST_LightFact)
	{
	hpstatus = 200;
	players[player].have_lights++;
	}

if (tpe == ST_HeavyFact)
	{
	hpstatus = 500;
	players[player].have_heavys++;
	}

if (tpe == ST_Windtrap)
	{
	hpstatus = 150;		// Windtraps are weak!
	players[player].have_powers++;
	}


if (flag == 0)
        {
         hpget = (hpstatus / 2);
        }

if (flag == 1)
        {
         int vier = hpstatus / 4;
         hpget = vier * 3;
        }

if (flag >= 2)
        {
         hpget = hpstatus;
        }

structure[res].tpe = tpe;
structure[res].dead_flag = 1;
structure[res].player = player;
structure[res].house = players[player].house;
structure[res].cell = cell;
structure[res].max_hitpoints = hpstatus;
structure[res].hitpoints = hpget;
structure[res].selected = 0;
structure[res].place_b = -1;    // No building to place
structure[res].facing = rnd(7); // Facing
return 0;               // Succes
}

void unselect_all()
{
 int i;
 for (i = 0; i < 250; i++)
        {
         structure[i].selected = 0;
        }

}
 /*
                        BUILD STUFF!!!!!!!!!!!  
 */

void do_building()
{
int i;
int price;
int perc;
int cur_player;
int slow;
int cntr;
int money_check;        // Must be higher than -1 or
                        // else we don't build further
 for (i = 0; i < 250; i++)
  {
   if ((builders[i].dead_flag != 0) && (builders[i].percent < 100))
        {
                 // Build god dammed stuff
        cur_player = builders[i].player;// The current player who builds this
        slow = builders[i].slowdowns;
        cntr = builders[i].counter;
         if (cntr <= slow)
          {
          builders[i].counter++;
          continue;     // Next one!
          }

        builders[i].counter = 0;
        price = builders[i].topay;      // The price to pay
        perc = price / 100;             // 1 Percent of price

        money_check = players[cur_player].must_credits;
        money_check -= perc;
        if (money_check >= 0)
                {
                players[cur_player].must_credits -= perc;

                // Ok we paid 1 percent of 100
                builders[i].percent++;
                } // Enough money
                
        } // Dead flag


  }

}

void do_structures()
{
// Draws all structures on map
int is_tur;	// Is building a turret or not?
int small; // for building size
int i;  // Loop control
int hs;	// House of structure
int tp;	// Type of structure
int px,py;	// X and Y of structure
int sizeX, sizeY; // Size of structure
int picture;    // What picture to use?
int sc;         // Cell
int selx1,selx2,sely1,sely2;    // Positions to select
int color;
int fc;	// Facing
clear_occupieds();       // Clear!
int health_drawit = 0;   // Not drawing health yet
int health_posx;
int health_posy;
int health_sizeX;
int health_healthx;

 for (i = 0; i < 250; i++)
  {
	// At start its unselected
  picture = ConstYard;    // Standard we draw a construction yard
  sizeX = 32;     // SizeX is standard 32
  sizeY = 32;   // And SizeY is standard 32
   if (structure[i].dead_flag == 1)
        {
        color = color_white;
	is_tur = 0;	// Not a turret
	small = 0;	// Not small
        
        sc = structure[i].cell;         // Cell of structure
        hs = structure[i].house;        // House?
        tp = structure[i].tpe;          // Type?
	fc = structure[i].facing;	// Facing
        px = x_cell(structure[i].cell); // Cell X and
        py = y_cell(structure[i].cell); // Y

        selx1 = (px - MX) * 16;
        sely1 = (py - MY) * 16;
        // At least 1 cell is occupied!
        map[sc].occupied_s = i;

        
        // Now we will check what type of structure it is.
        switch (tp)
         {
         case ST_ConstructionYard : picture = ConstYard;    break; // Construction yard
         case ST_Windtrap         : picture = WindTrap;     break; // Windtrap
         case ST_Refinery         : picture = Refinery;     break; // Refinery
         case ST_Silo             : picture = Silo;         break; // Silo
         case ST_Barrack          : picture = Barracks;     break; // Barracks
         case ST_Radar            : picture = Radar;        break; // Radar
         case ST_LightFact        : picture = LightFactory; break; // Light factory
         case ST_HeavyFact        : picture = HeavyFactory; break; // Heavy factory
         case ST_Starport         : picture = Starport;     break; // Starport
         case ST_Wor              : picture = WOR;          break; // WOR
         case ST_IX               : picture = IX;           break; // House of IX
         case ST_Hitech           : picture = HiTech;       break; // Hi-tech fac
         case ST_GunTurret        : picture = GunTurret;    break; // Gun turret
         case ST_RocketTurret     : picture = RocketTurret; break; // Rocket Turret
         case ST_Repair           : picture = Repair;       break; // Repair fac
         }
	

        if (picture == Refinery || picture == HeavyFactory || picture == Starport || picture == HiTech || picture == Repair)
                {
                sizeX = 48;    // Extra big
                map[sc + 2].occupied_s = i;
                map[sc + 66].occupied_s = i;

		if (picture == Starport)
			sizeY = 48;	// Super big

                }


	if (picture == GunTurret || picture == RocketTurret)
	{
	is_tur = 1;	// Its a turret
	sizeX = 16;	// Size 
	sizeY = 16;	// Size
	small = 1;	// Its small yeah!
	}

	if (small == 0)
	{
	map[sc + 1].occupied_s = i;
        map[sc + 64].occupied_s = i;
        map[sc + 65].occupied_s = i;
	}

        selx2 = selx1 + sizeX;
        sely2 = sely1 + sizeY;

        if (mouse_x >= selx1 && mouse_x <= selx2 && mouse_y >= sely1 && mouse_y <= sely2 && mouse_x < 511)
                     {
                    // Ok mouse is on this structure
                    // Show HP
                  health_drawit = 1;
                  float hp = structure[i].hitpoints;
                  float maxhp = structure[i].max_hitpoints;
                  int posx, posy;
                  float perc = sizeX / maxhp, healthx;
                  int healthx2 = 0;
                  // We need to know what 1 percent is of the total
                  // since one bar is sizeX big, we need to do
                  // Then we need to know how many hitpoints we have.
                  healthx = (perc * hp);	
                  healthx2 = (int)(healthx);
                  posx = (px - MX) * 16;
                  posy = (py - MY) * 16;
                  health_posx = posx;
                  health_posy = posy;
                  health_healthx = healthx2;
                  health_sizeX = sizeX;
		  if ((health_posy - 5) < 0)
			health_posy += 5;

                 if (mouse_b & 1)
                        {
                        unselect_all();
                        structure[i].selected = 1;
                        ingame_info = 0; // stop placing 
                        }
                }


        if (structure[i].player != Human)
                color = color_red;

        // Now draw structure
	if (is_tur == 0)
	        blit((BITMAP *)str[picture].dat, page, (hs * sizeX), 0, (px - MX) * 16, (py - MY) * 16, sizeX, sizeY);

	if (is_tur != 0)
	        blit((BITMAP *)str[picture].dat, page, (fc * sizeX), 0, (px - MX) * 16, (py - MY) * 16, sizeX, sizeY);

        if (structure[i].selected != 0)
                {
                rect(page, selx1, sely1, selx2-1, sely2-1, color);
                 if (tp == ST_ConstructionYard && structure[i].player == Human)
                        sidebar_info = BAR_BUILDINGS;

                 if (tp == ST_LightFact && structure[i].player == Human)
                        sidebar_info = BAR_UNITS;


                        sidebar_info_id = i;    // Pass our ID
                }
//        textprintf(page, font,  0, 100 , color_white, "DRAWING STRUCTURES");
        }
    }

if (health_drawit == 1)
 {
   rectfill(page, health_posx, health_posy-5, health_posx+health_sizeX-1, health_posy-1,color_black);
   rectfill(page, health_posx, health_posy-5, health_posx+health_healthx-1, health_posy-1, color_green);
   rect(page, health_posx, health_posy-5, health_posx+health_sizeX-1, health_posy-1,color_white);
 }

}


void clear_rest_s(int start)
{
int i;
icons_struct[start] = -1;
for (i = start; i <= 30; i++)
	{
	icons_struct[i] = -1;
	}

}


void set_houses()
{
int i;
 for (i = 0; i < 6; i++)
  {
  players[i].credits = 1000;
  players[i].must_credits = 1000;
  players[i].powerin = 0;
  players[i].powerout = 0;

	// Upgrades
  players[i].upgrade_4quad = 0;
  players[i].upgrade_4slab = 0;

	// Owner integers

  players[i].have_radars = 0;
  players[i].have_powers = 0;
  players[i].have_lights = 0;
  players[i].have_heavys = 0;
  players[i].have_refineries = 0;

  players[i].house = i;
  }

players[Human].house = Ordos;
players[CPU_1].house = Atreides;
}

void draw_minimap()
{
int i;			// Loop control
int color;
int dx, dy;
for (i = 0; i <= 4095;i++)
 {
	color = color_sandy;
        if (tile_nr(i) >= 1 && tile_nr(i) <= 16)
		color = color_rocky;

	if (tile_nr(i) >= 19 && tile_nr(i) <= 34)
		color = color_spicy;

        if (tile_nr(i) == 17)
                color = color_red;      // Dangerous, blooms are nasty

        if (tile_nr(i) == 18 || (tile_nr(i) >= 35 && tile_nr(i) <= 47))
                color = color_slab;     // Slabs and Walls

        if (map[i].occupied_s > -1)
                {
                // Draw house buildings
                int id;
                id = map[i].occupied_s;
                switch (structure[id].house)
                 {
                  case Atreides : color = color_blue; break;
                  case Ordos    : color = color_green; break;
                  case Harkonnen: color = color_red; break;
                  case Sardaukar: color = color_purple; break;
                  case Mercenary: color = color_brown; break;
                  case Fremen   : color = color_yellow; break;

                 }

                }
                

	dx = x_cell(i);
	dy = y_cell(i);  
	dx = dx * 2;
	dy = dy * 2;
                
	rect(page, 511 + dx, 352 + dy , 511 + dx + 1, 352 + dy + 1, color);

 }

rect (page, 511 + (MX * 2), 352 + (MY * 2), 511 + (MX * 2)+ 62, 352 + (MY * 2) + 58, color_white);
rect (page, 511 + (MX * 2) + 1, 352 + (MY * 2) + 1, 511 + (MX * 2) + 62 + 1, 352 + (MY * 2) + 58 + 1, color_white);


}


int on_left(int cell, int tile)
{
// Return 0 for NO, != 0 for YES
cell--;
cell = fix_cell(cell);
if (map[cell].tile == tile)
	return 1;

return 0;
}

int on_right(int cell, int tile)
{
// Return 0 for NO, != 0 for YES
cell++;
cell = fix_cell(cell);
if (map[cell].tile == tile)
	return 1;

return 0;
}

int on_up(int cell, int tile)
{
// Return 0 for NO, != 0 for YES
cell -= 64;
cell = fix_cell(cell);
if (map[cell].tile == tile)
	return 1;

return 0;
}

int on_down(int cell, int tile)
{
// Return 0 for NO, != 0 for YES
cell += 64;
cell = fix_cell(cell);
if (map[cell].tile == tile)
	return 1;

return 0;
}

int mouse_cell()
{
// Returns cell
// -1 = On Sidebar
// -2 = On Minimap
// -3 = On Icons list
// -4 = On UP_icons list button
// -5 = On DOWN_icons list button
int x;
int y;
if (mouse_x >= 544 && mouse_y >= 20 && mouse_x <= 640 && mouse_y <= 41)
	return -4;
if (mouse_x >= 544 && mouse_y >= 322 && mouse_x <= 640 && mouse_y <= 346)
	return -5;
if (mouse_x >= 546 && mouse_y >= 56 && mouse_x <= 640 && mouse_y <= 314)
	return -3;
if (mouse_x >= 500 && mouse_y >= 352)
	return -2;		// On minimap
if (mouse_x >= 500)
	return -1;		// On Sidebar


x = mouse_x / 16;
y = mouse_y / 16;
x = x + MX;
y = y + MY;
return make_cell(x,y);
}


int is_tile(int cl1, int cl2)
{
int flag = false;
if (map[cl1].tile == map[cl2].tile)
	flag = true;

return flag;
}


void smooth_terrain()
{
// Smooth the darn terrain
int i;		// Loop control
int left = 0;
int right = 0;
int up = 0;
int down = 0;
int maketile;


         for (i = 0; i <= 4095; i++)
                {
                map[i].pretile = map[i].tile;
                 if (map[i].tile >= 1 && map[i].tile <= 16)
                        {
                        map[i].pretile = 1;
                        map[i].tile = 1;
                        }
                 if (map[i].tile == 18)
                        map[i].pretile = 18;
                        
                 if (map[i].tile >= 19 && map[i].tile <= 34)
                        {
                        map[i].pretile = 19;
                        map[i].tile = 19;
                        }
                }

int wl;
for (i = 0; i <= 4095; i++)
 {

  if (map[i].tile == 1)
	{
	maketile = 1;			// Standard we have a solid rock plateau

	up = on_up(i, 1);	
	down = on_down(i, 1);
	left = on_left(i, 1);
        right = on_right(i, 1);

        // Slabs
        for (wl = 0; wl < 13; wl++)
        {
        if (up == 0)
          up = on_up(i, 18);       // And slabs
        if (down == 0)
                down = on_down(i, 18);
        if (left == 0)
                left = on_left(i, 18);
        if (right == 0)
                right = on_right(i, 18);
        }


        // Walls
        for (wl = 0; wl < 13; wl++)
        {
        if (up == 0)
          up = on_up(i, 35 + wl);       // And walls
        if (down == 0)
                down = on_down(i, 35 + wl);
        if (left == 0)
                left = on_left(i, 35 + wl);
        if (right == 0)
                right = on_right(i, 35 + wl);
        }

	// Now perform the test
	if ((left == 1) && (right == 1) && ( up == 1) && (down == 1))
		maketile = 1;

	if ((left == 0) && (right == 0) && ( up == 0) && (down == 0))
		maketile = 10;


	if ((left == 1) && (right == 1) && ( up == 0) && (down == 0))
		maketile = 11;

	if ((left == 0) && (right == 0) && ( up == 1) && (down == 1))
		maketile = 16;


	if ((left == 0) && (right == 1) && ( up == 0) && (down == 1))
		maketile = 6;

	if ((left == 0) && (right == 1) && ( up == 1) && (down == 0))
		maketile = 9;

	if ((left == 1) && (right == 0) && ( up == 0) && (down == 1))
		maketile = 8;

	if ((left == 1) && (right == 0) && ( up == 1) && (down == 0))
		maketile = 7;


	if ((left == 1) && (right == 1) && ( up == 1) && (down == 0))
		maketile = 5;

	if ((left == 1) && (right == 1) && ( up == 0) && (down == 1))
		maketile = 4;

	if ((left == 0) && (right == 1) && ( up == 1) && (down == 1))
		maketile = 2;

	if ((left == 1) && (right == 0) && ( up == 1) && (down == 1))
		maketile = 3;


	if ((left == 1) && (right == 0) && ( up == 0) && (down == 0))
		maketile = 12;

	if ((left == 0) && (right == 1) && ( up == 0) && (down == 0))
		maketile = 13;

	if ((left == 0) && (right == 0) && ( up == 1) && (down == 0))
		maketile = 14;

	if ((left == 0) && (right == 0) && ( up == 0) && (down == 1))
		maketile = 15;

		map[i].pretile = maketile;
	}

  if (map[i].tile == 19)
	{
	maketile = 19;			// Standard we have a solid rock plateau

	up = on_up(i, 19);	
	down = on_down(i, 19);
	left = on_left(i, 19);
        right = on_right(i, 19);

	// Now perform the test
	if ((left == 1) && (right == 1) && ( up == 1) && (down == 1))
		maketile = 19;

	if ((left == 0) && (right == 0) && ( up == 0) && (down == 0))
		maketile = 28;


	if ((left == 1) && (right == 1) && ( up == 0) && (down == 0))
		maketile = 29;

	if ((left == 0) && (right == 0) && ( up == 1) && (down == 1))
		maketile = 34;


	if ((left == 0) && (right == 1) && ( up == 0) && (down == 1))
		maketile = 24;

	if ((left == 0) && (right == 1) && ( up == 1) && (down == 0))
		maketile = 27;

	if ((left == 1) && (right == 0) && ( up == 0) && (down == 1))
		maketile = 26;

	if ((left == 1) && (right == 0) && ( up == 1) && (down == 0))
		maketile = 25;


	if ((left == 1) && (right == 1) && ( up == 1) && (down == 0))
		maketile = 23;

	if ((left == 1) && (right == 1) && ( up == 0) && (down == 1))
		maketile = 22;

	if ((left == 0) && (right == 1) && ( up == 1) && (down == 1))
		maketile = 20;

	if ((left == 1) && (right == 0) && ( up == 1) && (down == 1))
		maketile = 21;


	if ((left == 1) && (right == 0) && ( up == 0) && (down == 0))
		maketile = 30;

	if ((left == 0) && (right == 1) && ( up == 0) && (down == 0))
		maketile = 31;

	if ((left == 0) && (right == 0) && ( up == 1) && (down == 0))
		maketile = 32;

	if ((left == 0) && (right == 0) && ( up == 0) && (down == 1))
		maketile = 33;

		map[i].pretile = maketile;
	}


 }

for (i = 0; i <= 4095; i++)
	{
	map[i].tile = map[i].pretile;
	}

}

void clear_terrain(int tile)
{
int i;
 for (i = 0; i <= 4095; i++)
  {
   map[i].tile = tile;
  }

}


int fix_cell(int cell)
{
if (cell < 0)
	return 0;
if (cell > 4095)
	return 4095;

return cell;
}

int rnd(int range)
{
 return (random() % range);
}



int side4(int cell, int tile)
{
// Check at 4 sides for 'tile'
int flag = 0;
int cll;
cll = cell;
if (map[fix_cell(cll--)].tile == tile)
	flag++; 

if (map[fix_cell(cll++)].tile == tile)
	flag++; 

if (map[fix_cell(cll -= 64)].tile == tile)
	flag++; 

if (map[fix_cell(cll += 64)].tile == tile)
	flag++; 

return flag;
}


void thick_spots(int tpe)
{
int k;
 for (k = 0; k <= 4095; k++)
  {
	if (tpe == SPOT_ROCK)
	{
	 if (map[k].tile != 1)			// Found something else than rock
	 {
	   if (side4(k, 1) > 2)
		map[k].tile = 1;		// Seems enough rock around it so make this rock

	   if (side4(k, 1) == 2)		// Gamble, fifty fifty... rock or not?
		{
		 if (rnd(4) < 2) 
			map[k].tile = 1;
		}		


	 }	
	} // END IF TPE = ROCK

	if (tpe == SPOT_SPICE)
	{
	 if (map[k].tile != 19)			// Found something else than spice
	 {
	   if (side4(k, 19) > 2)
		map[k].tile = 19;		// Seems enough spice around it so make this spice

	   if (side4(k, 19) == 2)		// Gamble, fifty fifty... spice or not?
		{
		 if (rnd(4) < 2) 
			map[k].tile = 19;
		}		


	 }	
	} // END IF TPE = SPICE

  }	// End for


}


void make_spot(int cell, int tpe)
{
int dir;		// Direction
int j;			// Loop control
 for (j = 0; j < 400; j++)
  {
   dir = rnd(4);	// Random Dir
	switch(dir)
	 {
	  case 0 : cell--; break;
	  case 1 : cell++; break;
	  case 2 : cell -= 64; break;
	  case 3 : cell += 64; break;
	 }
   cell = fix_cell(cell);
   if (tpe == SPOT_SPICE)
	{
	if (map[cell].tile == 1)
		continue;		// Do not place the spice spot, priority is ROCK!
	}
   map[cell].tile = tpe;

  }

}

void add_blooms(int amount)
{
int i;
int done = 0;
while (done < amount)
{
  i = rnd(4095);
   if (map[i].tile == 0)
        {
        map[i].tile = 17;      // Spice bloom
        done++;
         }
    
} // WHILE

}
void make_random_map()
{
int i, j;
int spots = 5;
int fields = 5;
int spot1, spot2;
int spot;
int dif;
clear_terrain(0);
die_all_structures();   // Reset structures

 for (i = 0; i < spots; i++)
  {
   spot = rnd(4096);
   if (i == 0)
        spot1 = spot;
   else
   {
    spot2 = spot;
    if ((spot2-spot1) > dif)
        dif = (spot2-spot1);
   }

   make_spot(spot, SPOT_ROCK);
  }

// Spice fields
 for (i = 0; i < fields; i++)
  {
   spot = rnd(4096);
   make_spot(spot, SPOT_SPICE);
  }


thick_spots(SPOT_ROCK);
thick_spots(SPOT_ROCK);
thick_spots(SPOT_ROCK);
thick_spots(SPOT_SPICE);
thick_spots(SPOT_SPICE);

for (j = 0; j < 10; j++)
 {
  map[rnd(4096)].tile = 1;
 }

smooth_terrain();
add_blooms(rnd(10));

// Add 2 construction yards
add_structure(0,ST_ConstructionYard,spot1, 2);
add_structure(1,ST_ConstructionYard,spot2, 2);

}

/*
Cell calculations
*/
int x_cell(int cell)            			// Returns the X of the cell
{
 int tmpy = cell / 64;
 return (cell - (tmpy * 64));
}

int y_cell(int cell)            			// Return the Y of the cell
{
 return (cell / 64);       
}

int make_cell(int inx, int iny) 			// Make a cell of X and Y
{
 return (iny * 64) + inx;
}
 

int tile_nr(int cell)					// Give Tile NR of cell
{

return map[cell].tile;

}





void do_sidebar()					// Do sidebar stuff
{
int color;
int i;
  masked_blit((BITMAP *)graphcm[Sidebar].dat, page, 0, 0, 500, 0, 140, 480);
  draw_minimap();
  // Now draw money of player
	color = color_white;

  if (players[Human].must_credits > players[Human].credits)
		{
		color = color_green;
                players[Human].credits = players[Human].must_credits;
		}
  if (players[Human].must_credits < players[Human].credits)
		{
		color = color_yellow;
                players[Human].credits = players[Human].must_credits;
		}
  textprintf(page, font, 580, 5, color_black, "$%d", players[Human].credits);
  textprintf(page, font, 580, 6, color, "$%d", players[Human].credits);

	// Draw icons
	int icon;
	int start, end;
	int dix = 0;

/*

******************************************************************************
HANDLE UNITS LIST ON SIDEBAR!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
******************************************************************************

*/
	if (sidebar_info == BAR_UNITS)
		{
         int building = 0;      // Not yet
         int player_l;  // List
         building = is_building(sidebar_info_id);
         player_l = structure[sidebar_info_id].player;         

          if (building == 0 && (player_l == Human))            // IF
          {                     // NOT BUILDING!

            if (sidebar_scroll > max_scroll_unit)
                  sidebar_scroll = max_scroll_unit;

            if (sidebar_scroll < 0)
                  sidebar_scroll = 0;

           start = sidebar_scroll;
           end = start + 4;
           for (i = start; i < end; i++)
           {
              icon = icons_units[i];
              draw_icon_uni(icon, dix);
  	      dix++;
	     } // FOR
          } // IF


		}




/*

******************************************************************************
HANDLE STRUCTURES LIST ON SIDEBAR!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
******************************************************************************

*/



        // If we have selected a construction yard!
        if (sidebar_info == BAR_BUILDINGS)
         {                                                      // START
                // Are we building?
         int building = 0;      // Not yet
         int player_l;  // List
         building = is_building(sidebar_info_id);
         player_l = structure[sidebar_info_id].player;         
          if (building == 0 && (player_l == Human))            // IF
          {                     // NOT BUILDING!
            if (sidebar_scroll > max_scroll_building)
                  sidebar_scroll = max_scroll_building;

            if (sidebar_scroll < 0)
                  sidebar_scroll = 0;

           start = sidebar_scroll;
           end = start + 4;
           for (i = start; i < end; i++)
           {
            icon = icons_struct[i];

		    if (mouse_cell() == -3)
			{
				if (mouse_y > (60+(63*dix)) && mouse_y < (60+(63*dix))+64)
                                {
//                            textprintf(page, font, 0, 70, color_white, "Icon on Sidbar %d", icon);

				if (name_structure(icon) != "Unknown")
					{                                
					rectfill(page,400, (60+(63*dix))-1, 557, (60+(63*dix))+11, color_black);
     	                        rect(page,400, (60+(63*dix))-1, 557, (60+(63*dix))+11, color_white);
					textprintf(page, font, 403,(60+(63*dix))+2 , color_yellow,"%s", name_structure(icon));                         
					}
                                // Ok , when we click, we will build
                                if ((mouse_b & 1) && name_structure(icon) != "Unknown")
                                  add_builder(sidebar_info_id,-1,icon,Human); 

                                }
                        }

                    draw_icon_str(icon, dix);
		    dix++;
                                
           } // FOR

          }
          else
          { //NOT BUILDING!!!!!!!!

          // Draw the icon we build, under that a percent block
            icon = builders[sidebar_info_id].type_b;
            draw_icon_str(icon,0);
            int per = builders[sidebar_info_id].percent;
            int per2 = per;             // per2 = in percent (for IF )
            per = (int)(per * 0.64);    // Calculate from Percent to Pixel
            rectfill(page, 560, 126, 625, 138,color_black);     // Black back
            rectfill(page, 560, 126, 561+per, 138,color_blue);  // Percent complete
            rect(page, 560, 126, 625, 138, color_white);        // White border

            if (per2 >= 100)
              {
               // Draw place stuff
              draw_icon_str(ICON_Place, 3);
                              int tpe_b;
                              tpe_b = builders[sidebar_info_id].type_b;



				// If it is an UPGRADE button , then it is:
				if (tpe_b == ICON_Upgrade)
				{
                                builders[sidebar_info_id].dead_flag = 0; // done building
				// It seems that the player didnt upgrade anything yet
				// from the const yard, the first thing you can only
				// Upgrade is to 4SLAB, so thats what we gonna do
				players[Human].upgrade_4slab++;

				ingame_info = 0;	// We don't place anything yet on second thought...
				}


		    if (mouse_cell() == -3)
			{
                         if(mouse_b & 1)
                          {
                           if (mouse_y > (60+(63*3)) && mouse_y < (60+(63*3))+48)
                                {
                          

			   // Ok we're gonna place something
                              building_h = 2;
                              building_w = 2;
                              ingame_info = 1;
                              // Change the building h_w if nescesarry
                                if (tpe_b == ICON_Slab1 || tpe_b == ICON_Rturret || tpe_b == ICON_Gturret || tpe_b == ICON_Wall)
                                        {
                                        building_w = 1;
                                        building_h = 1;
                                        }

                                if (tpe_b == ICON_Refinery || tpe_b == ICON_Heavy_Factory || tpe_b == ICON_Hitech || tpe_b == ICON_Repair)
                                        building_w = 3;
			
				if (tpe_b == ICON_Starport)
					{
					building_w = 3;
					building_h = 3;
					}                                


                                 }

                          }
                        }
              }
                // Draw a CANCEL button, so we can cancel building at any time.
                // When pressed we will return all credits, simply PER2 * PRICE/100
            if (per2 > 1)
                 {
                 draw_icon_str(ICON_Cancel, 2);

              if (mouse_cell() == -3)
                    {
                  if (mouse_y > (60+(63*2)) && mouse_y < (60+(63*2))+48)
                        {

                        if (mouse_b & 1)
                                {
                                // Cancel!!!
                                // Total refund
                                int refund = 0;
                                int oneof = 0;
                                oneof = (int)(builders[sidebar_info_id].topay / 100);
                                refund = oneof * per2;
                                builders[sidebar_info_id].dead_flag = 0;
                                players[Human].must_credits += refund;
                                ingame_info = 0;        // We stop placing something
                                }

                        }
                    }
               } // We can only cancel when production has started

          } // WE DO BUILD!!

         }                                              // END OF BUILDINGS

/*

**************************************************************************
			END OF STRUCTURES CODE
**************************************************************************

*/



}

void players_think()					// Players think
{
int i;


// Fix any errors on map drawing here
// Has nothing to do with thinking of players
if (MX < 0)
	MX = 0;
if (MX > 32)
        MX = 32;

if (MY < 0)
	MY = 0;

if (MY > 34)
        MY = 34;

// End corrections

 for (i = 1; i <= 6; i++)
  {
 	  // ai_think(i);		// Ai must think...
	 
		
  } // FOR

}


int install_game()					// Install game
{
// This game runs in 640X480 mode , 24 bits
set_color_depth(16);    // 16 bits
if (set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0))	// 640X480 mode
	return 1;					// Oops something wend wrong

if (install_mouse() < 0)
		return 2;				// No mouse found


srandom(time(0));

install_keyboard();			// Install keyboard handler

graph01 = load_datafile("graph01.dat");
graphcm = load_datafile("graphcm.dat");
str = load_datafile("structs.dat");

page = create_bitmap(640,480);             // Drawing videopage

color_greenish = makecol(208,243,85);   // Greenish
color_slab = makecol(124,124,96);       // Slab
color_rocky = makecol(48, 48, 36);      // Minimap ROCKY
color_sandy = makecol(180, 120, 56);	// Minimap SANDY
color_white = makecol(255, 255, 255);   // White
color_black = makecol(0, 0, 0);         // Black
color_yellow = makecol(255,255,0);      // Yellow (credits IN)
color_green = makecol(0,255,0);         // Green (credits OUT and Ordos)
color_red = makecol(255,0,0);           // Harkonnen
color_spicy = makecol(184, 92, 32);     // Spicy
color_purple = makecol(183,5,181);      // Purple, Sardaukar
color_brown = makecol(183,55,5);        // Mercenary
color_blue = makecol(0,0,255);          // Atreides


mouse_status = MOUSE_NORMAL;

// Scrolling for sidebar
max_scroll_building = 0;
max_scroll_unit = 0;
max_scroll_air = 0;
sidebar_scroll = 0;
return 0;
}                       // END INSTALL GAME

void do_mouse()						// Do mouse stuff
{
// Do mouse stuff
int mouse_picture = Mouse1;	// Standard just the mouse
int mousex,mousey;

// Now we check for boundries
mousex = mouse_x;
mousey = mouse_y;


/*

'is moved:
IF DQBmouseLB = -1 AND tekenen = 0 THEN x1 = DQBmouseX: y1 = DQBmouseY
IF DQBmouseLB = -1 AND x1 <> -1 AND y1 <> -1 THEN x2 = DQBmouseX: y2 = DQBmouseY: tekenen = 1

IF tekenen = 1 THEN
'Maydraw now,
magscrollen = 1 'You may not scroll when you multi-select
IF x2 > 245 THEN x2 = 245        'Max for multiselect
IF x1 > 245 THEN x1 = 245
IF x2 > x1 AND y2 > y1 THEN DQBbox 1, x1, y1, x2, y2, white
IF x2 > x1 AND y2 < y1 THEN DQBbox 1, x1, y2, x2, y1, white
IF x2 < x1 AND y2 > y1 THEN DQBbox 1, x2, y1, x1, y2, white
IF x2 < x1 AND y2 < y1 THEN DQBbox 1, x2, y2, x1, y1, white
END IF

IF DQBmouseLB = 0 AND tekenen = 1 THEN tekenen = 0: magscrollen = 0: checkunits = 2  'reset all

*/

if ((mouse_b & 1) && mouse_draw_box == 0 && (mouse_x < 509))
	{
	x1 = mouse_x;
	y1 = mouse_y;
	}

if ((mouse_b & 1) && (x1 != -1) && (y1 != -1) && (mouse_x < 509))
	{
	x2 = mouse_x;
	y2 = mouse_y;
	mouse_draw_box = 1;
	}

if (mouse_draw_box == 1)
	{
	if (x1 > 509)
		x1 = 509;

	if (x2 > 509)
		x2 = 509;

	rect(page, x1, y1, x2, y2, color_white);
	}


if (!(mouse_b & 1))
	{
	mouse_draw_box = 0;
	}

// END OF MULTISELECTING


if (mouse_x == 0 && MX > 0 && mouse_draw_box == 0)
	{
	MX--;
	mouse_picture = MouseL;
	}

if (mouse_x == 639 && MX < 31 && mouse_draw_box == 0)
	{
	MX++;
	mouse_picture = MouseR;
	mousex -= 5;
	}

if (mouse_y == 0 && MY > 0 && mouse_draw_box == 0)
	{
	MY--;
	mouse_picture = MouseU;
	}

if (mouse_y == 479 && MY < 35 && mouse_draw_box == 0)
	{
	MY++;
	mouse_picture = MouseD;
	mousey -= 5;
	}

masked_blit((BITMAP *)graphcm[mouse_picture].dat, page, 0, 0,mousex, mousey, 11, 14);
int mc = mouse_cell();
if (mc >= 0)
       {
        // Mouse behaviour on battle field
        if (mouse_b & 2)
                {
                unselect_all(); // right click
                ingame_info = 0; // Stop placing something
                }
        if (mouse_b & 1)
                {
                 // If we place something...
                 if (ingame_info == 1 && placing_good >= 1)
                        {
                         // Place structure
                        int tpe;
                        int fl;
                        if (placing_good == 1)
                               fl = 0;  // good, but you only get 50%

                        if (placing_good == 2)

                               fl = 1;  // better, 75%

                        if (placing_good >= 3)
                               fl = 2;  // 100%


                        tpe = builders[sidebar_info_id].type_b;
                        builders[sidebar_info_id].dead_flag = 0; // Done!
                        tpe =  convert_icon2str(tpe);
                        add_structure(Human, tpe, mouse_cell(), fl);
                        ingame_info = 0;
                        placing_good = 0;
                        }

                }

       }

/*
textprintf(page, font, 0, 0, makecol(255,255,255), "Mouse cell : %d", mouse_cell());
textprintf(page, font, 0, 120, makecol(255,255,255), "Struct : %d", map[mc].occupied_s);

if (mc >= 0)
 textprintf(page, font, 0, 8, makecol(255,255,255), "Map Tile : %d", map[mc].tile);

if (mc == -1)
 textprintf(page, font, 0, 8, makecol(255,255,255), "On Sidebar");
*/
if (mc == -2)
  {
// textprintf(page, font, 0, 8, makecol(255,255,255), "On Minimap");

  int pX = ((mouse_x - 511) / 2);
  int pY = ((mouse_y - 352) / 2);

// textprintf(page, font, 0, 16, makecol(255,255,255), "%d %d", pX,pY );

	// When mouse click, then we should move to the swap to that position.
 	if (mouse_b & 1)	// Left Mouse Button Pressed
		{
		MX = pX - 16;
		MY = pY - 16;
		}

  }


if (mc == -4)
	{
//        textprintf(page, font, 0, 8, makecol(255,255,255), "On UP Button");
	  if (mouse_b & 1)
		  sidebar_scroll--;
	}

if (mc == -5)
	{
//        textprintf(page, font, 0, 8, makecol(255,255,255), "On DOWN Button");
	 if (mouse_b & 1)
		sidebar_scroll++;

	}


}       // END

void draw_terrain()
{
int x;				// Loop control
int y;				// Loop control
int cll;			// temp cell variable
int tile;			// variable holds tile nr
int picture;			// picture INT for data file
int startx, starty, endx, endy;	// Drawing positions
startx = MX;			// Calculate START x
starty = MY;			// START y
endx = startx + 32;		// END x
endy = starty + 30;		// END y

// Begin draw loops
for (y = starty; y < endy; y++)
 {
  for (x = startx; x < endx; x++)
   {

	cll = make_cell(x, y);	// Make a cell out of it
	tile = tile_nr(cll);	// Get tile sort

	switch (tile)
	 {
	  case 0 : picture = t0; break;	// Sand tile
            // Begin of rock tiles
	  case 1 : picture = t1; break;
	  case 2 : picture = t2; break;
	  case 3 : picture = t3; break;
	  case 4 : picture = t4; break;
	  case 5 : picture = t5; break;
	  case 6 : picture = t6; break;
	  case 7 : picture = t7; break;
	  case 8 : picture = t8; break;
	  case 9 : picture = t9; break;
	  case 10 : picture = t10; break;
	  case 11 : picture = t11; break;
	  case 12 : picture = t12; break;
	  case 13 : picture = t13; break;
	  case 14 : picture = t14; break;
	  case 15 : picture = t15; break;
	  case 16 : picture = t16; break;
            // End of rock tiles
          case 17 : picture = t17; break;       // Bloom
            // Begin of GENS (walls, slabs)
          case 18 : picture = slab1; break;
            // Begin of SPICE
	  case 19 : picture = s1; break; 
	  case 20 : picture = s2; break; 
	  case 21 : picture = s3; break; 
	  case 22 : picture = s4; break; 
	  case 23 : picture = s5; break; 
	  case 24 : picture = s6; break; 
	  case 25 : picture = s7; break; 
	  case 26 : picture = s8; break; 
	  case 27 : picture = s9; break; 
	  case 28 : picture = s10; break; 
	  case 29 : picture = s11; break; 
	  case 30 : picture = s12; break; 
	  case 31 : picture = s13; break; 
	  case 32 : picture = s14; break; 
	  case 33 : picture = s15; break; 
	  case 34 : picture = s16; break; 
	  // End of SPICE
          // NR 35 >= && <= 47 is Wall
          // Begin of Concrete Wall
          case 35 : picture = w1; break;
          case 36 : picture = w2; break;
          case 37 : picture = w3; break;
          case 38 : picture = w4; break;
          case 39 : picture = w5; break;
          case 40 : picture = w6; break;
          case 41 : picture = w7; break;
          case 42 : picture = w8; break;
          case 43 : picture = w9; break;
          case 44 : picture = w10; break;
          case 45 : picture = w11; break;
          case 46 : picture = w12; break;
          case 47 : picture = w13; break;
          // End of Concrete Wall
     
	  default : picture = t0;	// Default we draw sand
	 } // Switch

        if (tile >= 35 && tile <= 47)
        handle_walls(cll);                // Wall!!!

	blit((BITMAP *)graph01[picture].dat, page, 0, 0, (x - MX) * 16, (y - MY) * 16, 16, 16);

   } // FOR X
  } // For Y

} // End of DRAW_TERRAIN

int main()
{
allegro_init();
if (install_game() != 0)
	{
	allegro_exit();
	
 	 switch (install_game()) 
          {
	 case 1 : cout << "Resolution could not been set on your system.\n"; break;
	 case 2 : cout << "Mouse not found"; break;
	 default : cout << "Cannot initialize game.\n";
	  }
	return 0;
	}

if (!graph01 || !graphcm)
 {
  allegro_exit();
  cout << "Unable to load all data files.\n";
  return 0;
 }
text_mode(-1);		// Transparant text

set_houses();		// Reset values
make_random_map();	// Make random map



ingame_info = 0;                // We don't know yet if we gonna
                                // place a structure yet
while (game_running == true)
 {
	clear(page);
	handle_lists();
        placing_good = 0;               // At start we have an invalid spot
        mouse_status = MOUSE_NORMAL;    // At start we have a normal mouse
        sidebar_info = BAR_NONE;        // At start we don't know yet
	players_think();		// Let players think
        smooth_terrain();               // Smooth terrain before drawin it
	draw_terrain();			// Draw terrain
        do_building();                  // Handle building actions
        do_structures();                // Draw structures (and AI)
        // do_units();                  // Draw units (and AI)

        if (ingame_info == 1)           // Are we gonna place something?
          placing_good = place_structure(building_w, building_h);   // Yep!


        do_sidebar();                   // Draw Sidebar
	do_mouse();			// Draw mouse and handle actions
	

	if (key[KEY_ESC])
		game_running = false;
	if (key[KEY_F11])
		make_random_map();
        if (key[KEY_F12])
                save_bmp("screen.bmp",page,pal);

	if (key[KEY_A])
		players[Human].must_credits += 100;

	if (key[KEY_Z])
		players[Human].must_credits -= 100;

	if (key[KEY_Q])
		techlevel++;

	if (key[KEY_W])
		techlevel--;

                


//  textprintf(page, font, 0, 50, color_white, "Scroll_Sidebar: %d", sidebar_scroll);
//  textprintf(page, font, 0, 58, color_white, "Scroll_Sidebar MAX struct: %d", max_scroll_building);
//  textprintf(page, font, 0, 0, color_white, "INGAME_INFO %d", ingame_info);
//  textprintf(page, font, 0, 10, color_white, "PLACING_GOOD %d", placing_good);
  textprintf(page, font, 0, 10, color_white, " %d", max_scroll_unit);

  textprintf(page, font, 0, 0, color_white, "TECHLEVEL %d", techlevel);

  blit(page, screen, 0, 0, 0, 0, 640, 480);
}

return 0;
}

